from AnimatedWordCloud.Animator.AllocationCalculator.StaticAllocationCalculator.StaticAllocationCalculator import (
    allocate,
    allocate_all,
)

__all__ = [
    "allocate",
    "allocate_all",
]
