from AnimatedWordCloud.Animator.AllocationCalculator.StaticAllocationCalculator.StaticAllocationStrategies.MagneticAllocation.MagneticAllocation import (
    MagneticAllocation,
)

__all__ = [
    "MagneticAllocation",
]
